﻿namespace Phoenix.Model.Management
{
    using System;

    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Provides features to manage the doctor types.
    /// </summary>
    public class DoctorTypeModel : Manager<DoctorType>
    {
        #region Fields

        private IDoctorTypeInputPresenter presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorTypeModel"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public DoctorTypeModel(IDoctorTypeInputPresenter presenter)
            : base(RepositoryFactory.Get<IDoctorTypeRepository>())
        {
            this.presenter = presenter;
            this.presenter.Inserted += presenter_Inserted;
            this.presenter.Updated += presenter_Updated;
            this.presenter.Deleted += view_Deleted;
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Indicates if the specified doctor type is already in the repository.
        /// Check only the name of the doctor type.
        /// </summary>
        /// <param name="doctorType">Type of the doctor.</param>
        /// <returns></returns>
        public bool Exists(DoctorType doctorType)
        {
            var repository = this.Repository as IDoctorTypeRepository;
            string msg = string.Format("The cast into the 'IDoctorTypeRepository' failed because the repository is a {0}", this.Repository.GetType().Name);
            Assert.IsTrue((repository != null), msg);

            return (repository.Search(doctorType).Count > 0)
                ? true
                : false;
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns><c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c></returns>
        public override bool ValidateData()
        {
            float parsed;
            bool result = true;
            if (this.presenter.ItemToManage.Title == string.Empty)
            {
                this.presenter.SetError(DoctorTypeFailOn.Title, "The specialisation couldn't be empty.".Translate());
                result = false;
            }
            if (float.TryParse(this.presenter.ItemToManage.Title, out parsed))
            {
                this.presenter.SetError(DoctorTypeFailOn.Title, "The specialisation couldn't be a number.".Translate());
                result = false;
            }
            if (float.TryParse(this.presenter.ItemToManage.Remark, out parsed))
            {
                this.presenter.SetError(DoctorTypeFailOn.RemarkIsNumber, "The remark for the specialisation couldn't be a number.".Translate());
                result = false;
            }
            return result;
        }

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected override void Delete(DoctorType item)
        {
            base.Delete(item);
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Process to execute when a insertion is raised.
        /// </summary>
        /// <param name="item">The item to insert.</param>
        protected override void Insert(DoctorType item)
        {
            base.Insert(item);
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Process to execute when an update is raised.
        /// </summary>
        /// <param name="item">The item to update.</param>
        protected override void Update(DoctorType item)
        {
            base.Update(item);
            AppContext.NotifyDataChanged();
        }

        private void presenter_Inserted(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                base.Insert(this.presenter.ItemToManage);
                this.presenter.Close();
            }
        }

        private void presenter_Updated(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                base.Update(this.presenter.ItemToManage);
                this.presenter.Close();
            }
        }

        void view_Deleted(object sender, EventArgs e)
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(DoctorTypeFailOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                base.Delete(this.presenter.ItemToManage);
                this.presenter.Close();
            }
        }

        #endregion Methods
    }
}