﻿namespace Phoenix.View.Frame
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;

    using Ascend.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Management;
    using Phoenix.Presenter.Sessions;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;
    using Phoenix.View.Input;
    using Phoenix.View.Management;

    using WeifenLuo.WinFormsUI.Docking;

    /// <summary>
    /// Represent the side sideMenu to administrate the whole application.
    /// </summary>
    public partial class SideMenu : TranslateableForm
    {
        #region Fields

        private CalendarBox calendarBox;
        private DockPanel dockPanel;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SideMenu"/> class.
        /// </summary>
        public SideMenu(DockPanel dockPanel)
        {
            InitializeComponent();
            this.TranslateForm();
            this.dockPanel = dockPanel;
            this.Logger = LogManager.GetLogger(typeof(SideMenu));
        }

        #endregion Constructors

        #region Enumerations

        /// <summary>
        /// All the menu contained in the SideMenu
        /// </summary>
        public enum NavigationPane
        {
            /// <summary>
            /// Sessions
            /// </summary>
            Sessions,
            /// <summary>
            /// Meetings
            /// </summary>
            Meetings,
            /// <summary>
            /// Statistics
            /// </summary>
            Statistics,
            /// <summary>
            /// Administration
            /// </summary>
            Administration,
        }

        #endregion Enumerations

        #region Events

        /// <summary>
        /// Occurs when the user asked a deletion of an item.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> Deleted;

        /// <summary>
        /// Occurs when this DockContent is hidden.
        /// </summary>
        public event EventHandler Hidden;

        /// <summary>
        /// Occurs when the user clicks on the "search" button of the openedSessions manager.
        /// </summary>
        public event EventHandler SearchPatientClicked;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the list of the managers of the opened session.
        /// </summary>
        /// <value>A list of SessionGui.</value>
        public SessionCollection SessionsCollection
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new open sessions in the menu.
        /// </summary>
        /// <param name="session">The session.</param>
        /// <returns><c>True</c> if the session is open. Otherwise,
        /// if a session is already opened, returns <c>False</c></returns>
        public bool Add(Session session)
        {
            if (!this.SessionsCollection.Contains(session))
            {
                session.Modified += session_Modified;
                session.PatientSelected += session_PatientSelected;
                InsertInTreeView(session);
                this.SessionsCollection.Add(session);
                return true;
            }
            else
            {
                string msg = "A session is already open for '{0}'";
                MessageBoxes.Warning(msg.Translate(session.Patient.ToString()));
                return false;
            }
        }

        /// <summary>
        /// Adds a new NavigationPane to the SideMenu.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="text">The text.</param>
        /// <param name="control">The control.</param>
        public void AddMenu(string key, string text, Control control)
        {
            var pane = new NavigationPanePage();
            pane.Text = text;
            pane.Key = key;
            control.Dock = DockStyle.Fill;
            pane.Controls.Add(control);
            this.nav_SideMenu.NavigationPages.Add(pane);
        }

        /// <summary>
        /// Determines whether  this SideMenu contains the specified patient in the opened sessions.
        /// </summary>
        /// <param name="patient">The patient.</param>
        /// <returns>
        /// 	<c>true</c> if [contains] [the specified patient]; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(Patient patient)
        {
            return (from session in this.SessionsCollection
                    where session.Patient.ID == patient.ID
                    select session).Count() > 0;
        }

        /// <summary>
        /// Refreshes all the opened managers.
        /// </summary>
        public void RefreshGui()
        {
            this.tv_Sessions.Nodes.Clear();
            foreach (var session in this.SessionsCollection)
            {
                this.RefreshSessions(session);
                this.InsertInTreeView(session);
            }
            this.tv_Sessions.ExpandAll();
        }

        /// <summary>
        /// Refreshes the session to remove all the closed sessions.
        /// </summary>
        public void RefreshSessions()
        {
            this.SessionsCollection.RemoveClosedSessions();
            this.tv_Sessions.SuspendLayout();
            this.tv_Sessions.Nodes.Clear();
            this.InsertInTreeView(this.SessionsCollection);
            this.tv_Sessions.ResumeLayout();
        }

        /// <summary>
        /// Removes the specified openedSessions from the menu and close this openedSessions.
        /// </summary>
        /// <param name="session">The session to remove.</param>
        public void Remove(Session session)
        {
            if (!this.SessionsCollection.Contains(session))
            {
                string msg = string.Format("No session exists for '{0}'", session.Patient);
                MessageBoxes.Warning(msg.Translate(session.Patient));
                base.Logger.Warn(msg);
                return;
            }

            base.Logger.Debug(string.Format("Remove session '{0}'", session.Patient.ID));
            this.UnsubscribeEvents(session);
            this.SessionsCollection.Remove(session);

            foreach (TreeNode node in this.tv_Sessions.Nodes)
            {
                Session otherSession = node.Tag as Session;
                if (otherSession == session)
                {
                    Logger.Debug(string.Format("Remove node with session '{0}'", ((Session)node.Tag).Patient.ID));
                    tv_Sessions.Nodes.Remove(node);
                    return;
                }
            }
        }

        /// <summary>
        /// Selects the specified NavigationPane.
        /// </summary>
        /// <param name="navigationPane">The menu.</param>
        public void Select(NavigationPane navigationPane)
        {
            switch (navigationPane)
            {
                case NavigationPane.Sessions:
                    this.nav_SideMenu.SelectNavigationPage(this.nSessions);
                    break;
                case NavigationPane.Meetings:
                    this.nav_SideMenu.SelectNavigationPage(this.nMeetings);
                    break;
                default:
                    Assert.FailsOnEnumeration(navigationPane);
                    break;
            }
        }

        private void btn_SearchSession_Click(object sender, EventArgs e)
        {
            OnSessionClicked();
        }

        private void InsertInTreeView(Session session)
        {
            string name = string.Format("{0} {1}", session.Patient.FirstName, session.Patient.LastName);
            TreeNode item = new TreeNode(name);
            item.ImageKey = "Root";
            item.Tag = session;

            foreach (var manager in session.Managers)
            {
                TreeNode subitem = new TreeNode(manager.Description.Translate());
                subitem.ImageKey = manager.IconValue.ToString();
                subitem.Tag = manager;
                item.Nodes.Add(subitem);
            }

            this.tv_Sessions.Nodes.Add(item);
            this.tv_Sessions.ExpandAll();
        }

        private void InsertInTreeView(SessionCollection sessions)
        {
            foreach (Session item in sessions)
            {
                this.InsertInTreeView(item);
            }
        }

        private void nav_SideMenu_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.nav_SideMenu.SelectedNavigationPage == this.nMeetings)
            {
                this.calendarBox.Show(this.dockPanel);
            }
            else
            {
                this.calendarBox.Hide();
            }
        }

        private void OnDeleted(Patient patient)
        {
            this.Deleted.Raise(this, new EventArgs<Patient>(patient));
        }

        private void OnSessionClicked()
        {
            this.SearchPatientClicked.Raise(this);
        }

        private void RefreshSessions(Session session)
        {
            foreach (var manager in session.Managers)
            {
                if (manager != null) manager.RefreshData();
            }
        }

        private void session_Modified(object sender, EventArgs e)
        {
            this.SessionsCollection.Refresh();
            this.RefreshSessions();
        }

        private void session_PatientSelected(object sender, EventArgs<Patient> e)
        {
            Session session = new Session(e.Data);
            this.Add(session);
        }

        private void SideMenu_DockStateChanged(object sender, EventArgs e)
        {
            if (this.DockState == DockState.Hidden)
            {
                this.Hidden.Raise(this);
            }
        }

        private void SideMenu_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Logger.Debug("Side Menu raised FormClosing...");
            base.Hide();
            this.Hidden.Raise(this);
            e.Cancel = true;
        }

        private void SideMenu_Load(object sender, EventArgs e)
        {
            AppContext.Context.Add("SideMenu", this);
            AppContext.SessionUpdated += (sender2, e2) => this.RefreshGui();

            this.calendarBox = new CalendarBox();

            this.SessionsCollection = new SessionCollection();

            this.Text = "Side menu".Translate();
            this.btn_SearchSession.Text = "Search...".Translate();

            this.nSessions.Text = "Sessions".Translate();
            this.nMeetings.Text = "Meetings".Translate();
        }

        private void tsmi_CloseManagers_Click(object sender, EventArgs e)
        {
            Session session = tv_Sessions.SelectedNode.Tag as Session;

            session.Managers.ForEach(manager =>
            {
                if (manager is DockContent)
                {
                    var dock = manager as DockContent;
                    dock.Hide();
                }
            });
        }

        private void tsmi_CloseSession_Click(object sender, EventArgs e)
        {
            var dr = MessageBoxes.Question("Do you want to close this session?");

            if (dr == DialogResult.Yes)
            {
                Session session = tv_Sessions.SelectedNode.Tag as Session;
                Assert.IsNotNull(session, "session");

                this.Remove(session);
            }
        }

        private void tsmi_Collapse_Click(object sender, EventArgs e)
        {
            this.tv_Sessions.CollapseAll();
        }

        private void tsmi_Expand_Click(object sender, EventArgs e)
        {
            this.tv_Sessions.ExpandAll();
        }

        private void tsmi_NextMeetings_Click(object sender, EventArgs e)
        {
            var session = tv_Sessions.SelectedNode.Tag as Session;
            Assert.IsNotNull(session, "session");

            var history = new NextMeetingsPdf();
            history.BuildPdf(session.Patient);
        }

        private void tv_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            e.Node.SelectedImageKey = e.Node.ImageKey;
        }

        private void tv_Sessions_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                tv_Sessions.SelectedNode = tv_Sessions.GetNodeAt(e.X, e.Y);
            }
        }

        private void tv_Sessions_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Point clickedPoint = new Point(e.X, e.Y);
                TreeNode clickedNode = tv_Sessions.GetNodeAt(clickedPoint);
                if (clickedNode == null || !(clickedNode.Tag is Session))
                    return;
                else
                {
                    // Convert from Tree coordinates to Screen coordinates
                    Point screenPoint = tv_Sessions.PointToScreen(clickedPoint);
                    // Convert from Screen coordinates to Form coordinates
                    Point formPoint = this.PointToClient(screenPoint);
                    // Show context menu
                    cms_ContextSession.Show(this, formPoint);
                }
            }
        }

        private void tv_Sessions_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            DockContent form = e.Node.Tag as DockContent;
            if (form != null) form.Show(this.dockPanel);
        }

        private void UnsubscribeEvents(Session session)
        {
            session.PatientSelected -= session_PatientSelected;
            session.Modified -= session_Modified;
        }

        #endregion Methods
    }
}