﻿namespace Phoenix.View.Helpers.Controls
{
    using System;
    using System.Windows.Forms;

    using Phoenix.Helpers.Misc;

    /// <summary>
    /// A titled ComboBox
    /// </summary>
    /// <remarks>This class is sealed because it overrides the property DataBindings whose is not virtual. </remarks>
    public sealed partial class TitledComboBox : UserControl, ITitledControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TitledComboBox"/> class.
        /// </summary>
        public TitledComboBox()
        {
            InitializeComponent();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when ComboBox selection changed.
        /// </summary>
        public event EventHandler<EventArgs<object>> SelectionChanged;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the data bindings for the control.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// A <see cref="T:System.Windows.Forms.ControlBindingsCollection"/> that contains the <see cref="T:System.Windows.Forms.Binding"/> objects for the control.
        /// </returns>
        public new ControlBindingsCollection DataBindings
        {
            get { return this.comboBox.DataBindings; }
        }

        /// <summary>
        /// Gets or sets the width of the label's column.
        /// </summary>
        /// <value>The width of the label.</value>
        public float LabelWidth
        {
            get
            {
                return this.tableLayoutPanel.ColumnStyles[0].Width;
            }
            set
            {
                this.tableLayoutPanel.ColumnStyles[0].Width = value;
            }
        }

        /// <summary>
        /// Selects the item in the ComboBox with the specified index.
        /// </summary>
        public int SelectedIndex
        {
            get { return this.comboBox.SelectedIndex; }
            set
            {
                this.comboBox.SelectedIndex = value;
            }
        }

        /// <summary>
        /// Gets or sets the selected item.
        /// </summary>
        /// <value>The selected item.</value>
        public object SelectedItem
        {
            get
            {
                return this.comboBox.SelectedItem;
            }
            set
            {
                this.comboBox.SelectedItem = value;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return this.label.Text;
            }
            set
            {
                this.label.Text = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new data binding.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>
        public void AddDataBinding(object dataSource, string dataMember)
        {
            if (this.comboBox.DataSource == null) return;

            this.comboBox.DataBindings.Add("SelectedValue", dataSource, dataMember, false, DataSourceUpdateMode.OnPropertyChanged, "Not selected");
        }

        /// <summary>
        /// Binds the specified data source to the combobox.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="displayMember">The display member for the ComboBox.</param>
        public void Bind(object dataSource, string displayMember)
        {
            this.comboBox.DataSource = null;
            this.comboBox.DataSource = dataSource;
            this.comboBox.DisplayMember = displayMember;
        }

        /// <summary>
        /// Binds the specified data source.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="displayMember">The display member for the ComboBox.</param>
        /// <param name="valueMember">The value member for the ComboBox.</param>
        public void Bind(object dataSource, string displayMember, string valueMember)
        {
            this.Bind(dataSource, displayMember);
            this.comboBox.ValueMember = valueMember;
        }

        /// <summary>
        /// Clears the data binding.
        /// </summary>
        public void ClearDataBinding()
        {
            this.comboBox.DataBindings.Clear();
        }

        /// <summary>
        /// Selects the specified item with the specified name.
        /// </summary>
        /// <param name="itemName">Name of the item.</param>
        public void SelectText(string itemName)
        {
            int idx = this.comboBox.FindStringExact(itemName);
            this.comboBox.SelectedIndex = idx;
        }

        private void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.comboBox.SelectedItem != null) this.OnSelectionChanged(this.comboBox.SelectedItem);
        }

        private void OnSelectionChanged(object selection)
        {
            this.SelectionChanged.Raise(this, new EventArgs<object>(selection));
        }

        #endregion Methods
    }
}