﻿namespace Phoenix.View.Helpers
{
    using System;

    using Phoenix.Helpers.Winforms;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Translation;

    /// <summary>
    /// Provides prefilled message boxes.
    /// </summary>
    public static class PredifinedMsgBoxes
    {
        #region Methods

        /// <summary>
        /// Indicates to the user the process has finished.
        /// </summary>
        /// <param name="usage">The usage (added/removed/updated an item).</param>
        /// <param name="hasSucceeded">if set to <c>true</c> the process has succeeded, otherwise <c>true</c>.</param>
        public static void InfoProcessExecuted(Usage usage, bool hasSucceeded)
        {
            if (hasSucceeded)
                InfoOnItemProcessed(usage);
            else
                InfoOnItemProcessedFailure(usage);
        }

        /// <summary>
        /// Warns user that the processed item already exists.
        /// </summary>
        public static void ItemAlreadyExist()
        {
            string msg = "The item already exist!".Translate();
            MessageBoxes.Information(msg);
        }

        /// <summary>
        /// Warn the user that the feature he tries to execute is not yet implemented.
        /// </summary>
        public static void NotYetImplemented()
        {
            MessageBoxes.Information("This feature is not yet implemented. It will be released soon!".Translate());
        }

        /// <summary>
        /// Display an error message for unexpected error with the message of
        /// the specified exception.
        /// </summary>
        /// <param name="ex">The exception to log in the message box.</param>
        public static void UnexpectedError(Exception ex)
        {
            string msg = string.Format("An unexpected error occured: {0}{1}{0}For further information, please read the logs."
                , Environment.NewLine
                , ex.Message);
            MessageBoxes.Error(msg);
        }

        /// <summary>
        /// Display an error message for unexpected error.
        /// </summary>        
        public static void UnexpectedError()
        {
            string msg = string.Format("An unexpected error occured!{0}For further information, please read the logs."
                , Environment.NewLine);
            MessageBoxes.Error(msg);
        }

        /// <summary>
        /// Warns user with a message box to highlight invalid criterion(s).
        /// </summary>
        public static void WarnInvalidValues()
        {
            string msg = "One or some value you entered is/are invalid. "
                + "Hover the mouse on the exclamation mark to see the error.".Translate();
            MessageBoxes.Information(msg);
        }

        /// <summary>
        /// Warns when user didn't activate any criterion on a searchd.
        /// </summary>
        public static void WarnOnNoCriterionActivated()
        {
            string msg = "Please activate at least one criterion to process the search!".Translate();
            MessageBoxes.Exclamation(msg);
        }

        /// <summary>
        /// Warns the user the item he/she wants to delete is referenced somewhere else in the domain.
        /// </summary>
        public static void WarnOnReferencedItem()
        {
            var msg = "Impossible to delete this item because it is referenced somewhere else.";
            MessageBoxes.Warning(msg);
        }

        /// <summary>
        /// Warns user about an unexpected error and invites him.her to see the logs.
        /// </summary>
        public static void WarnUnexpectedError()
        {
            string msg = "Can't execute the search! See logs for further information.".Translate();
            MessageBoxes.Error(msg);
        }

        /// <summary>
        /// Indicates the user the item was correctly processed.
        /// </summary>
        /// <param name="usage">The process just done.</param>
        private static void InfoOnItemProcessed(Usage usage)
        {
            string msg = "The item is correctly {0}";
            string process = string.Empty;

            switch (usage)
            {
                case Usage.Insert:
                    process = "inserted";
                    break;
                case Usage.Update:
                    process = "updated";
                    break;
                case Usage.Delete:
                    process = "deleted";
                    break;
                default:
                    break;
            }

            msg = string.Format(msg, process);
            MessageBoxes.Information(msg.Translate());
        }

        /// <summary>
        /// Indicates the user the item was correctly processed.
        /// </summary>
        /// <param name="usage">The process just done.</param>
        private static void InfoOnItemProcessedFailure(Usage usage)
        {
            string msg = "Impossible to {0} the item";
            string process = string.Empty;

            switch (usage)
            {
                case Usage.Insert:
                    process = "inserted";
                    break;
                case Usage.Update:
                    process = "updated";
                    break;
                case Usage.Delete:
                    process = "deleted";
                    break;
                default:
                    break;
            }

            msg = string.Format(msg, process);
            MessageBoxes.Information(msg.Translate());
        }

        #endregion Methods
    }
}