﻿namespace Phoenix.View.Input
{
    using System;
    using System.Reflection;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Translation;

    /// <summary>
    /// This manager is the basis to make a manager to manager
    /// domain item with two fields and a ComboBox for instance name, remark
    /// and type.
    /// </summary>
    public partial class BasicInputWithComboBox : TranslateableForm
    {
        #region Fields

        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BasicInputWithComboBox"/> class.
        /// </summary>
        /// <param name="itemToManage">A default instance for item to manage.</param>
        /// <param name="dataMember1">The data member1 is the name of the property of the specified itemToManage to link with the GUI.</param>
        /// <param name="dataMember2">The data member2 is the name of the property of the specified itemToManage to link with the GUI.</param>
        protected BasicInputWithComboBox(object itemToManage, string dataMember1, string dataMember2)
        {
            this.Logger = LogManager.GetLogger(typeof(BasicInput));
            InitializeComponent();
            this.Usage = Usage.Insert;
            this.Text1 = "Title:".Translate();
            this.Text2 = "Remark".Translate();
            this.Text3 = "Type:".Translate();
            this.Bind(itemToManage, dataMember1, dataMember2);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BasicInputWithComboBox"/> class.
        /// </summary>
        /// <param name="itemToManage">A default instance for item to manage.</param>
        /// <param name="dataMember1">The data member1 is the name of the property of the specified itemToManage to link with the GUI.</param>
        /// <param name="dataMember2">The data member2 is the name of the property of the specified itemToManage to link with the GUI.</param>
        /// <param name="comboBoxMember">The combo box member.</param>
        protected BasicInputWithComboBox(object itemToManage, string dataMember1, string dataMember2, string comboBoxMember)
            : this(itemToManage, dataMember1, dataMember2)
        {
            this.Logger = LogManager.GetLogger(typeof(BasicInput));
            InitializeComponent();
            this.Usage = Usage.Insert;
            this.Text1 = "Title:".Translate();
            this.Text2 = "Remark".Translate();
            this.Text3 = "Type:".Translate();
            this.Bind(itemToManage, dataMember1, dataMember2, comboBoxMember);
        }

        /// <summary>       
        /// Just for the designer
        /// </summary>
        private BasicInputWithComboBox()
        {
            this.InitializeComponent();
        }

        #endregion Constructors

        #region Enumerations

        /// <summary>
        /// Indicates on which control the error should be displayed
        /// </summary>
        protected enum On
        {
            /// <summary>
            /// The error is displayed on the TextBox for the Value1.
            /// </summary>
            Value1,
            /// <summary>
            /// The error is displayed on the TextBox for the Value2.
            /// </summary>
            Value2,
            /// <summary>
            /// The error is displayed on the ComboBox.
            /// </summary>
            Value3
        }

        #endregion Enumerations

        #region Events

        /// <summary>
        /// Occurs when the gui is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the value of the first label.
        /// </summary>
        /// <value>The text of the label.</value>
        public string Text1
        {
            get
            {
                return this.lbl_Value1.Text;
            }
            set
            {
                this.lbl_Value1.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the value of the second label.
        /// </summary>
        /// <value>The text of the label.</value>
        public string Text2
        {
            get
            {
                return this.lbl_Value2.Text;
            }
            set
            {
                this.lbl_Value2.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the value of the third label.
        /// </summary>
        /// <value>The text of the label.</value>
        public string Text3
        {
            get
            {
                return this.lbl_Value3.Text;
            }
            set
            {
                this.lbl_Value3.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the usage of this manager.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get { return this.usage; }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Insert:
                        this.btn_Add.Text = "Add".Translate();
                        break;
                    case Usage.Update:
                        this.btn_Add.Text = "Update".Translate();
                        break;
                    case Usage.Delete:
                        this.btn_Add.Text = "Delete".Translate();
                        this.tb_Value1.ReadOnly
                            = this.tb_Value2.ReadOnly
                            = true;
                        this.cb_Value3.Enabled = false;
                        break;
                    default:
                        this.Logger.WarnFormat("Unexpected usage:'{0}'", value);
                        break;
                }
            }
        }

        /// <summary>
        /// Gets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        protected object Item
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the selected combo box item.
        /// </summary>
        /// <value>The selected combo box item.</value>
        protected object SelectedComboBoxItem
        {
            get { return this.cb_Value3.SelectedItem; }
            set { this.cb_Value3.SelectedItem = value; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Binds the specified item to manage into the two textboxes.
        /// The datamembers will link the textbox with the member of the specified
        /// itemToManage that has the specified name
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        /// <param name="dataMember1">The data member1.</param>
        /// <param name="dataMember2">The data member2.</param>
        public virtual void Bind(object itemToManage, string dataMember1, string dataMember2)
        {
            string propertyName = "Text";
            this.Item = itemToManage;

            this.tb_Value1.DataBindings.Clear();
            this.tb_Value2.DataBindings.Clear();

            this.tb_Value1.DataBindings.Add(propertyName, this.Item, dataMember1);
            this.tb_Value2.DataBindings.Add(propertyName, this.Item, dataMember2);
        }

        /// <summary>
        /// Binds the specified item to manage into the two textboxes.
        /// The datamembers will link the textbox with the member of the specified
        /// itemToManage that has the specified name
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        /// <param name="dataMember1">The data member1.</param>
        /// <param name="dataMember2">The data member2.</param>
        /// <param name="comboBoxMember">The combo box member.</param>
        public virtual void Bind(object itemToManage, string dataMember1, string dataMember2, string comboBoxMember)
        {
            this.Bind(itemToManage, dataMember1, dataMember2);

            PropertyInfo property = itemToManage.GetType().GetProperty(comboBoxMember);
            var selectedItem = property.GetValue(itemToManage, null);

            this.cb_Value3.SelectedItem = selectedItem;
        }

        /// <summary>
        /// Clears the errors.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets the combo box's data source.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>        
        public void SetDataSource(object dataSource, string dataMember)
        {
            this.cb_Value3.DataSource = dataSource;
            this.cb_Value3.DisplayMember = dataMember;
        }

        /// <summary>
        /// Deletes the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Delete(object itemToManage)
        {
            var ex = new NotImplementedException("Please implement this method in the child class");
            this.Logger.Error(ex);
            throw ex;
        }

        /// <summary>
        /// This method is called whenever the Load event is called.
        /// </summary>
        protected virtual void FormLoading()
        {
            this.OnRefreshed();
        }

        /// <summary>
        /// Inserts the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Insert(object itemToManage)
        {
            var ex = new NotImplementedException("Please implement this method in the child class");
            this.Logger.Error(ex);
            throw ex;
        }

        /// <summary>
        /// Called to trigger the Refreshed event.
        /// </summary>
        protected void OnRefreshed()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        protected virtual void RefreshData()
        {
            //Does nothing by default
        }

        /// <summary>
        /// Sets the error on value1.
        /// </summary>
        /// <param name="onValue">The on value.</param>
        /// <param name="message">The message.</param>
        protected void SetError(On onValue, string message)
        {
            Control control = new Control();
            switch (onValue)
            {
                case On.Value1:
                    control = this.tb_Value1;
                    break;
                case On.Value2:
                    control = this.tb_Value2;
                    break;
                case On.Value3:
                    control = this.cb_Value3;
                    break;
                default:
                    Assert.FailsOnEnumeration(onValue);
                    break;
            }
            this.ep_Error.SetError(control, message);
        }

        /// <summary>
        /// Updates the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Update(object itemToManage)
        {
            var ex = new NotImplementedException("Please implement this method in the child class");
            this.Logger.Error(ex);
            throw ex;
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>Returns <c>true</c> if the data is valid; otherwise <b>false</b></returns>
        protected virtual bool ValidateData()
        {
            return true;
        }

        private void BasicManagerWithComboBox_Load(object sender, EventArgs e)
        {
            this.tb_Value1.Select();
            if (this.Usage == Usage.Insert)
                this.FormLoading();
        }

        private void btn_Add_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Insert:
                    if (this.ValidateData())
                    {
                        this.Insert(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:

                    if (this.ValidateData())
                    {
                        this.Update(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:

                    if (this.ValidateData())
                    {
                        this.Delete(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        #endregion Methods
    }
}