﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Manage drugs
    /// </summary>
    public class DrugInput : BasicInputWithComboBox, IDrugInputPresenter
    {
        #region Fields

        private const string COMBOBOX_MEMBER = "Type";
        private const string VALUE1 = "Title";
        private const string VALUE2 = "Remark";

        private DrugModel presenter;
        private List<DrugType> types = new List<DrugType>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DrugInput"/> class.
        /// </summary>
        public DrugInput()
            : base(new Drug(), VALUE1, VALUE2)
        {
            this.presenter = new DrugModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user asked to delete an Drug
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user asked to insert an Drug
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when user asked to update an Drug
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Drug ItemToManage
        {
            get
            {
                var drug = this.Item as Drug;
                drug.Type = this.SelectedComboBoxItem as DrugType;
                return drug;
            }
            set
            {
                var temp = value as Drug;
                this.OnRefreshed();
                this.Bind(temp, VALUE1, VALUE2, COMBOBOX_MEMBER);
            }
        }

        /// <summary>
        /// Gets or sets the types of drug.
        /// </summary>
        /// <value>The types.</value>
        public List<DrugType> Types
        {
            get
            {
                return this.types;
            }
            set
            {
                this.types = value;
                this.SetDataSource(value, "Title");
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Sets an error message on validation failed.
        /// </summary>
        /// <param name="fail">Indicates where the validation failed.</param>
        /// <param name="message">The message to display.</param>
        public void SetError(DrugFailedOn fail, string message)
        {
            switch (fail)
            {
                case DrugFailedOn.NoName:
                    this.SetError(On.Value1, message);
                    break;
                case DrugFailedOn.NoType:
                    this.SetError(On.Value3, message);
                    break;
                case DrugFailedOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
        }

        /// <summary>
        /// Deletes the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Delete(object itemToManage)
        {
            this.Deleted.Raise(this);
        }

        /// <summary>
        /// Inserts the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Insert(object itemToManage)
        {
            this.Inserted.Raise(this);
        }

        /// <summary>
        /// Updates the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Update(object itemToManage)
        {
            this.Updated.Raise(this);
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// Returns <c>true</c> if the data is valid; otherwise <b>false</b>
        /// </returns>
        protected override bool ValidateData()
        {
            return this.presenter.ValidateData();
        }

        #endregion Methods
    }
}