﻿namespace Phoenix.View.Input
{
    using System;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter.Exceptions;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers.Controls;

    /// <summary>
    /// Input of a user
    /// </summary>
    public partial class UserInput : TranslateableForm, IUserInputPresenter
    {
        #region Fields

        private UserModel model;
        private Usage usage = Usage.Insert;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="UserInput"/> class.
        /// </summary>
        public UserInput()
        {
            InitializeComponent();
            this.model = new UserModel(this);
            this.Usage = Usage.Insert;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a user action occured.
        /// </summary>
        public event EventHandler<EventArgs<Usage>> ActionOccured;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public User ItemToManage
        {
            get
            {
                return this.bs_User.Current as User;
            }
            set
            {
                this.bs_User.DataSource = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this manager is in read only mode.
        /// </summary>
        /// <value><c>true</c> if [read only]; otherwise, <c>false</c>.</value>
        public bool ReadOnly
        {
            get
            {
                return (this.Usage == Usage.Display || this.Usage == Usage.Delete);
            }
        }

        /// <summary>
        /// Gets or sets the usage.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get { return this.usage; }
            set
            {
                this.usage = value;
                switch (this.usage)
                {
                    case Usage.Display:
                        this.btn_OK.Visible = false;
                        this.EnableControls(false);
                        this.btn_Cancel.Text = "Close".Translate();
                        break;
                    case Usage.Insert:
                        this.ItemToManage = new User();
                        this.EnableControls(true);
                        this.btn_OK.Visible = true;
                        this.btn_OK.Text = "Insert".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    case Usage.Update:
                        this.btn_OK.Visible = true;
                        this.EnableControls(true);
                        this.btn_OK.Text = "Update".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    case Usage.Delete:
                        this.btn_OK.Visible = true;
                        this.EnableControls(false);
                        this.btn_OK.Text = "Delete".Translate();

                        this.btn_Cancel.Text = "Cancel".Translate();
                        break;
                    default:
                        Assert.FailsOnEnumeration(this.usage);
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        private void bs_User_CurrentChanged(object sender, EventArgs e)
        {
            this.bs_Address.DataSource = null;
            this.bs_Practice.DataSource = null;

            var user = this.bs_User.Current as User;
            if (user == null) throw new BindingException("Impossible to bind neither the practice address not the main practice ti GUI");

            this.bs_Address.DataSource = user.MainPractice.Address;
            this.bs_Practice.DataSource = user.MainPractice;
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            this.OnActionOccured();
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }

        private void btn_SetPassword_Click(object sender, EventArgs e)
        {
            var form = new PasswordForm();
            var dr = form.ShowDialog();
            if (dr != System.Windows.Forms.DialogResult.OK) return;

            var user = this.bs_User.Current as User;

            if (user == null) throw new BindingException("Impossible to set the password to the user because of a binding error");
            user.Encrypt(form.Password);
            this.btn_SetPassword.Text = "Password set";
        }

        private void EnableControls(bool value)
        {
            foreach (var control in this.flp_Panel.Controls)
            {
                if (control is ITitledControl)
                    (control as ITitledControl).Enabled = value;
            }
        }

        private void OnActionOccured()
        {
            this.ActionOccured.Raise(this, new EventArgs<Usage>(this.Usage));
        }

        private void ttb_LastName_Validating(object sender, System.ComponentModel.CancelEventArgs e)
        {
            this.ttb_LastName.Value = this.ttb_LastName.Value.ToUpper();
        }

        #endregion Methods
    }
}