﻿namespace Phoenix.Presenter.Search.Criterion
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;

    /// <summary>
    /// Display a list of criterion.
    /// </summary>
    public partial class CriterionControlCollection : UserControl, ICollection<ICriterionControl>
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CriterionControlCollection"/> class.
        /// </summary>
        public CriterionControlCollection()
        {
            InitializeComponent();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the count of activated criterion.
        /// </summary>
        /// <value>The activated criterion count.</value>
        public int ActivatedCriterionCount
        {
            get
            {
                int count = 0;
                foreach (Control item in this.flp_Panel.Controls)
                {
                    CriterionControl criterion = item as CriterionControl;
                    if (criterion != null)
                    {
                        count += (criterion.IsActivated)
                            ? 1
                            : 0;
                    }
                }
                return count;
            }
        }

        /// <summary>
        /// Gets the number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <value></value>
        /// <returns>The number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.</returns>
        public int Count
        {
            get { return this.flp_Panel.Controls.Count; }
        }

        /// <summary>
        /// Gets a value indicating whether the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.
        /// </summary>
        /// <value></value>
        /// <returns>true if the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only; otherwise, false.</returns>
        public bool IsReadOnly
        {
            get { return this.flp_Panel.Controls.IsReadOnly; }
        }

        #endregion Properties

        #region Indexers

        /// <summary>
        /// Gets the <see cref="CriterionControl"/> with the specified name.
        /// </summary>
        /// <value></value>
        public CriterionControl this[string name]
        {
            get
            {
                foreach (Control item in this.flp_Panel.Controls)
                {
                    CriterionControl criterion = item as CriterionControl;
                    Assert.IsTrue(!string.IsNullOrEmpty(criterion.CriterionName), "A criterion should have a name!");

                    if (criterion != null && criterion.CriterionName == name)
                        return criterion;
                }
                return null;
            }
        }

        #endregion Indexers

        #region Methods

        /// <summary>
        /// Adds a criterion into the control.
        /// </summary>
        /// <param name="criterion">The criterion.</param>
        public void Add(ICriterionControl criterion)
        {
            if (criterion.IsControl)
            {
                this.flp_Panel.Controls.Add((Control)criterion);
            }
            else
            {
                string msg = string.Format("Impossible to add the new criterion: type {0} is not a control."
                    , criterion.GetType().Name);
                throw new ArgumentException(msg);
            }
        }

        /// <summary>
        /// Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only. </exception>
        public void Clear()
        {
            this.flp_Panel.Controls.Clear();
        }

        /// <summary>
        /// Determines whether the <see cref="T:System.Collections.Generic.ICollection`1"/> contains a specific value.
        /// </summary>
        /// <param name="item">The object to locate in the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param>
        /// <returns>
        /// true if <paramref name="item"/> is found in the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false.
        /// </returns>
        public bool Contains(ICriterionControl item)
        {
            Control ctrl = item as Control;
            if (ctrl.IsNotNull())
                return this.flp_Panel.Controls.Contains(ctrl);
            else
                return false;
        }

        /// <summary>
        /// Copies the elements of the <see cref="T:System.Collections.Generic.ICollection`1"/> to an <see cref="T:System.Array"/>, starting at a particular <see cref="T:System.Array"/> index.
        /// </summary>
        /// <param name="array">The one-dimensional <see cref="T:System.Array"/> that is the destination of the elements copied from <see cref="T:System.Collections.Generic.ICollection`1"/>. The <see cref="T:System.Array"/> must have zero-based indexing.</param>
        /// <param name="arrayIndex">The zero-based index in <paramref name="array"/> at which copying begins.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// 	<paramref name="array"/> is null.</exception>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// 	<paramref name="arrayIndex"/> is less than 0.</exception>
        public void CopyTo(ICriterionControl[] array, int arrayIndex)
        {
            this.flp_Panel.Controls.CopyTo(array, arrayIndex);
        }

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<ICriterionControl> GetEnumerator()
        {
            foreach (Control item in this.flp_Panel.Controls)
            {
                ICriterionControl criterion = item as ICriterionControl;
                if (criterion.IsNotNull())
                    yield return criterion;
            }
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        /// <summary>
        /// Removes the first occurrence of a specific object from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <param name="item">The object to remove from the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param>
        /// <returns>
        /// true if <paramref name="item"/> was successfully removed from the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false. This method also returns false if <paramref name="item"/> is not found in the original <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </returns>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.</exception>
        public bool Remove(ICriterionControl item)
        {
            Control ctrl = item as Control;
            if (ctrl.IsNotNull())
            {
                this.flp_Panel.Controls.Remove(ctrl);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Checks the validity of the values entered into the criterions.
        /// </summary>
        public bool ValidateCriterions()
        {
            bool ret = true;
            foreach (Control item in this.flp_Panel.Controls)
            {
                ICriterionControl criterion = item as ICriterionControl;
                if (criterion != null)
                {
                    ret = (criterion.ValidateCriterions())
                        ? ret
                        : false;
                }
            }
            return ret;
        }

        private void CriterionControlCollection_Load(object sender, EventArgs e)
        {
            this.SetCriterionSize();
        }

        private void flp_Panel_Resize(object sender, EventArgs e)
        {
            this.SetCriterionSize();
        }

        private void SetCriterionSize()
        {
            this.SuspendLayout();
            int width = (this.flp_Panel.VerticalScroll.Visible)
                ? this.flp_Panel.Width - 25
                : this.flp_Panel.Width - 10;

            foreach (Control item in this.flp_Panel.Controls)
            {
                item.Width = width;
            }
            this.ResumeLayout();
        }

        #endregion Methods
    }
}