﻿namespace Phoenix.Presenter.Search
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Search;
    using Phoenix.Presenter.Interfaces.Interfaces;
    using Phoenix.Presenter.Search.Criterion;
    using Phoenix.Translation;
    using Phoenix.View.Properties;

    /// <summary>
    /// Provides search box to find Doctors.
    /// </summary>
    public class DoctorSearchBox : SearchBox, ISearchBox<Doctor>
    {
        #region Fields

        private const string FIRSTNAME = "FirstName";
        private const string LASTNAME = "LastName";
        private const string SPECIALITY = "Speciality";

        private DoctorSearcher searcher;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorSearchBox"/> class.
        /// </summary>
        public DoctorSearchBox()
        {
            this.Text = "Search doctor";
            this.Logger = LogManager.GetLogger(typeof(DoctorSearchBox));
            this.Load += DoctorSearchBox_Load;

            #region Columns

            base.AddListViewColumn("c_Name", "Name");
            base.AddListViewColumn("c_Surname", "Surname");

            #endregion

            #region Criterions

            int tabindex = -1;
            this.searcher = new DoctorSearcher((this));

            ComboBoxCriterion speciality = new ComboBoxCriterion();
            speciality.Text = SPECIALITY.Translate();
            speciality.CriterionName = SPECIALITY;
            speciality.IsActivated = true;
            speciality.TabIndex = ++tabindex;
            speciality.Dock = DockStyle.Fill;
            this.Criterions.Add(speciality);

            TextCriterion surname = new TextCriterion();
            surname.Text = "Last name:".Translate();
            surname.CriterionName = LASTNAME;
            surname.IsActivated = false;
            surname.TabIndex = ++tabindex;
            surname.Dock = DockStyle.Fill;
            base.Criterions.Add(surname);

            TextCriterion name = new TextCriterion();
            name.Text = "First name:".Translate();
            name.CriterionName = FIRSTNAME;
            name.IsActivated = false;
            name.TabIndex = ++tabindex;
            name.Dock = DockStyle.Fill;
            base.Criterions.Add(name);

            #endregion

            this.DefaultView = (SearchBox.SearchBoxView)Enum.Parse(typeof(SearchBox.SearchBoxView), Settings.Default.SearchBox_DefaultView);
            this.SetSplitterDistance();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when an item is selected on the GUI.
        /// </summary>
        public event EventHandler<EventArgs<Doctor>> ItemSelected;

        /// <summary>
        /// Occurs when the user fire a search in the GUI
        /// </summary>
        public event EventHandler<EventArgs<Doctor>> Searching;

        #endregion Events

        #region Methods

        /// <summary>
        /// Fills the specified items into the GUI.
        /// </summary>
        /// <param name="items">The items to load into the SearchBox.</param>
        public void Fill(IList<Doctor> items)
        {
            List<ListViewItem> lvitems = new List<ListViewItem>();
            this.lv_Result.SuspendLayout();
            {
                this.ClearItemsFromListView();
                foreach (Doctor item in items)
                {
                    ListViewItem lvitem = new ListViewItem(item.LastName);
                    ListViewItem.ListViewSubItem firstnameItem = new ListViewItem.ListViewSubItem(lvitem, item.FirstName);

                    try
                    {
                        base.SetImage(ItemType.Doctor, lvitem);
                    }
                    catch (ArgumentException ex)
                    {
                        base.Logger.Warn(ex.Message, ex);
                        MessageBoxes.Warning(ex.Message);
                    }

                    lvitem.SubItems.Add(firstnameItem);
                    lvitem.Tag = item;
                    lvitem.Group = this.GetGroup(item.Specialisation.Title);
                    lvitems.Add(lvitem);
                }
                this.AddListViewItems(lvitems);
                this.AutoResizeListView();
                this.DeactivateProgressBar();
            }
            this.lv_Result.ResumeLayout();
        }

        /// <summary>
        /// Called when a Patient search is fired.
        /// </summary>
        protected override void OnSearch()
        {
            try
            {
                this.Criterion = GetCriterion();
                this.ActivateProgressBar();
                this.Searching.Raise(this, new EventArgs<Doctor>(this.Criterion as Doctor));
            }
            catch (Exception ex)
            {
                LogAndDisplayOnSearchError(ex);
            }
        }

        /// <summary>
        /// Called when a Patient selection is fired.
        /// </summary>
        protected override void OnSelection()
        {
            try
            {
                if (this.Searching != null)
                    this.ItemSelected.Raise(this, new EventArgs<Doctor>((Doctor)this.SelectedItem));
            }
            catch (Exception ex)
            {
                this.LogAndDisplayOnSelectError(ex);
            }
        }

        /// <summary>
        /// Provide custom checking before processing click in the search button.
        /// If this methods returns false, no further process for the click on search.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the check succeed. Otherwise returns <c>False</c>
        /// </returns>
        protected override bool ValidateData()
        {
            return true;
        }

        void DoctorSearchBox_Load(object sender, EventArgs e)
        {
            ComboBoxCriterion criterion = base.Criterions[SPECIALITY] as ComboBoxCriterion;
            if (criterion == null)
                throw new ArgumentException(string.Format("Impossible to find the criterion '{0}'", SPECIALITY));
            criterion.DataSource = searcher.DoctorTypes;
        }

        private Doctor GetCriterion()
        {
            Doctor p = new Doctor();
            CriterionControl ctrl;
            if ((ctrl = this.Criterions[FIRSTNAME]) != null
                && ctrl.IsActivated)
            {
                p.FirstName = ctrl.Value as string;
            }

            if ((ctrl = this.Criterions[LASTNAME]) != null
                && ctrl.IsActivated)
            {
                p.LastName = ctrl.Value as string;
            }

            if ((ctrl = this.Criterions[SPECIALITY]) != null
                && ctrl.IsActivated)
            {
                p.Specialisation = ctrl.Value as DoctorType;
            }

            return p;
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            //
            // DoctorSearchBox
            //
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(496, 593);
            this.Name = "DoctorSearchBox";
            this.Load += new System.EventHandler(this.DoctorSearchBox_Load);
            this.ResumeLayout(false);
            this.PerformLayout();
        }

        #endregion Methods
    }
}