﻿namespace Phoenix.Helpers.Pattern
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Helpers.Misc;

    /// <summary>
    /// With this tools, you can iterate a list and raise an event every time
    /// the current item changes. It's meant to be used in the GUI to make data binding.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class Iterator<T>
    {
        #region Fields

        private int index = 0;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Iterator&lt;T&gt;"/> class.
        /// </summary>
        public Iterator()
        {
            this.Collection = new List<T>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Iterator&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="collection">The collection to be managed by the iterator.</param>
        public Iterator(IList<T> collection)
        {
            this.Collection = collection;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the current item of the iterator changed.
        /// </summary>
        public event EventHandler Moved;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the current item of the list.
        /// </summary>
        /// <value>The current.</value>
        public T Current
        {
            get
            {
                if (this.index < 0) this.index = 0;
                if (this.index >= this.Collection.Count) this.index = this.Collection.Count - 1;
                return this.Collection[index];
            }
        }

        /// <summary>
        /// Gets the internal collection managed by the iterator.
        /// </summary>
        /// <value>The collection.</value>
        protected IList<T> Collection
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Moves the next item in the list.
        /// </summary>
        /// <returns></returns>
        public bool MoveNext()
        {
            var result = (++index >= this.Collection.Count);
            if (result) this.Moved.Raise(this);
            return result;
        }

        /// <summary>
        /// Moves the previous item in the list.
        /// </summary>
        /// <returns></returns>
        public bool MovePrevious()
        {
            var result = (--index < 0);
            if (result) this.Moved.Raise(this);
            return result;
        }

        /// <summary>
        /// Resets this the current item on the first one.
        /// </summary>
        public void Reset()
        {
            this.index = 0;
            this.Moved.Raise(this);
        }

        #endregion Methods
    }
}